### Model file for Data-Driven Ratio Optimization 
### This .mod file is used as the model for running Bisection Algorithm with Interval Compaction (IC) 
### and for IC + Prior Compaction (PC) algorithm 
### To run IC+PC, please adjust the upper bound of beta (betaU) in the .run file 

#--------------------------------------------------------------------------------------------------
# Define Sets, Parameters, and Variables
#--------------------------------------------------------------------------------------------------

set M;          # set of available assets in universe
set N;          # set of historical joint return scenarios
set K = 1..12;
param OutReturn{K,M}; 

param p0{N} default 1/card(N);    # probability of each scenario
param Y{N,M};                     # return of asset m in scenario n

param beta;     # threshold value (decision var - fixed number)
param betaL;    # lower bound of beta in bisection search
param betaU;    # upper bound of beta in bisection search
param err;      # threshold gap value for beta  
param Iter default 0;     # count the number of iterations in bisection
param IterCap default 10000;
param TotalIter;
set IterSet = 0..IterCap;
param betaIt{IterSet} default 0.5;
param betaItL{IterSet} default 0;
param betaItU{IterSet} default 1;

#########################################################################################
var x{M} >=0;       # capital weight invested in asset m;
var gamma >=0;      # variable in support function;
var w >=0;          # variable in conjugate function;
var kappa;          # variable in conjugate function;

###########################
var z{N} >=0;
var y{N};           # dual variable in support function, y[i];
var Rx{N};
param UR{i in N}:= max{m in M}(Y[i,m]);
param LR{i in N}:= min{m in M}(Y[i,m]);
#var v{j in N} >=-UR[j]/beta , <=-LR[j]/beta;    
var v{N};
# variable in conjugate function, v[j];
####################################################

var TLx{N};
var TRx{N} >=0;
var TCx{N};
#var pnew{N}; # probabilities with new constructed distribution;
#var pi{N,N}; # pi{i,j} joint probabilities for transforming;
param dw{i in N, j in  N} = sum{m in M} abs(Y[i,m] - Y[j,m]);  
# Wasserstein distance between any pair return realizations
param B = max{i in N, j in N} dw[i,j]/card(M); # diameter of support space
param q default 0.95;
param theta = B*sqrt(2/card(N)*log(1/(1-q))); # convergence rate
#param theta default 0.10;
		  
param xfea{M};
		  
#--------------------------------------------------------------------------------------------------
# Define the Feasibility Model/Problem DDR: 
# Check if the model is feasible under the current parameters/solutions
#--------------------------------------------------------------------------------------------------

minimize DDRObj:
    0;
		
subject to DDRSet1:
    gamma*theta + (sum{i in N} y[i]*p0[i]) + w/4 <= 0;

subject to DDRSet2{j in N}:
    (TCx[j])^2 + (TLx[j])^2 <= (TRx[j])^2;	
	
subject to DDRSet3{j in N}:
	z[j] = v[j] + (1/beta)*Rx[j];		
	
subject to DDRSet4{i in N, j in N}:
    (sum{m in M} abs(Y[j,m] - Y[i,m]))*gamma + y[i] >= v[j];	
	
subject to DDRSet5:
    sum{m in M} x[m] == 1;
	
subject to DDRSet6:
    gamma >= 0;	

subject to DDRSet7:
    w >= 0;	
	
subject to DDRSet8{j in N}:
    TLx[j] == 0.5*(z[j] - w);	

subject to DDRSet9{j in N}:
    TRx[j] == 0.5*(z[j] + w);	

subject to DDRSet10{j in N}:
    TCx[j] == Rx[j] - kappa; 	
	
subject to DDRSet11{j in N}:
    TRx[j] >= 0; 	
	
subject to DDRSet12{j in N}:
    Rx[j] == sum{m in M} Y[j,m]*x[m];	


#--------------------------------------------------------------------------------------------------
# Define the Model/Problem to Check beta
# Check if the current solution/constraint is binding with beta for improvement 
#--------------------------------------------------------------------------------------------------
param xp{M} default 1/card(M);
param PortReturn{j in N} = sum{m in M} Y[j,m]*xp[m];
param vC{N};
param wC;
param kappaC;
param RxC{j in N} = sum{m in M} Y[j,m]*xp[m] ;
param TCxC{j in N} = RxC[j] - kappaC;
var BetaCheck >=0;        # default 0.5;
var TLxC{j in N};     # = 0.5*(vC[j] + BetaCheck*RxC[j] - wC);
var TRxC{j in N};     # = 0.5*(vC[j] + BetaCheck*RxC[j] + wC);
var BetaC;

minimize CheckObj:
    BetaCheck;

subject to CheckSet1{j in N}:
    (TCxC[j])^2 + (TLxC[j])^2 <= (TRxC[j])^2;	
	
subject to CheckSet2{j in N}:
	vC[j] + BetaCheck*RxC[j] >= 0;	
	
subject to CheckSet3{j in N}:
    TLxC[j] >= 0.5*(vC[j] + BetaCheck*RxC[j] - wC);	

subject to CheckSet4{j in N}:
    TRxC[j] >= 0.5*(vC[j] + BetaCheck*RxC[j] + wC);	

subject to CheckSet5{j in N}:
    TLxC[j] >= 0;

subject to CheckSet6{j in N}:
    TRxC[j] >= 0;	
	
		