### Model file for Data-Driven Ratio Optimization 
### This .mod file is used as the model for running Standard Bisection Algorithm (SBA) 
### and for SBA + Prior Compaction (PC) algorithm 
### To run SBA+PC, please adjust the upper bound of beta (betaU) in the .run file 

#--------------------------------------------------------------------------------------------------
# Define Sets, Parameters, and Variables
#--------------------------------------------------------------------------------------------------

set M;          # set of available assets in universe
set N;          # set of historical joint return scenarios

set K = 1..12;
param OutReturn{K,M}; 

param p0{N} default 1/card(N);    # probability of each scenario
param Y{N,M};                     # return of asset m in scenario n

param beta;     # threshold value (decision var - fixed number)
param betaL;    # lower bound of beta in bisection search
param betaU;    # upper bound of beta in bisection search
param err;      # threshold gap value for beta  
param Iter default 0; 		# number of iterations in bisection
param IterCap default 10000;
param TotalIter;
set IterSet = 0..IterCap;
param betaIt{IterSet} default 0.5;
param betaItL{IterSet} default 0;
param betaItU{IterSet} default 1;

#########################################################################################
var x{M} >=0;       # capital weight invested in asset m;
var gamma >=0;      # variable in support function;
var w >=0;          # variable in conjugate function;
var kappa;          # variable in conjugate function;

####################################################
var z{N} >=0;
var y{N};           # dual variable in support function, y[i];
var Rx{N};
param UR{i in N}:= max{m in M}(Y[i,m]);
param LR{i in N}:= min{m in M}(Y[i,m]);
#var v{j in N} >=-UR[j]/beta , <=-LR[j]/beta;
var v{N};    
# variable in conjugate function, v[j];
####################################################

var TLx{N};
var TRx{N} >=0;
var TCx{N};
#var pnew{N}; # probabilities with new constructed distribution;
#var pi{N,N}; # pi{i,j} joint probabilities for transforming;
param dw{i in N, j in  N} = sum{m in M} abs(Y[i,m] - Y[j,m]);  
# Wasserstein distance between any pair return realizations
param B = max{i in N, j in N} dw[i,j]/card(M); # diameter of support space
param q default 0.95;
param theta = B*sqrt(2/card(N)*log(1/(1-q))); # convergence rate
#param theta default 0.05;
		  
#--------------------------------------------------------------------------------------------------
# Define the Feasibility Model: Check if the model is feasible under the current parameters/solutions
#--------------------------------------------------------------------------------------------------

minimize RandomObjConstant: 
      0;
		
subject to ConstraintSet1:
    gamma*theta + (sum{i in N} y[i]*p0[i]) + w/4 <= 0;

subject to ConstraintSet2{j in N}:
    (TCx[j])^2 + (TLx[j])^2 <= (TRx[j])^2;	
	
subject to ConstraintSet3{j in N}:
	z[j] = v[j] + (1/beta)*Rx[j];		
	
subject to ConstraintSet4{i in N, j in N}:
    (sum{m in M} abs(Y[j,m] - Y[i,m]))*gamma + y[i] >= v[j];	
	
subject to ConstraintSet5:
    sum{m in M} x[m] == 1;
	
subject to ConstraintSet8{j in N}:
    TLx[j] == 0.5*(z[j] - w);	

subject to ConstraintSet9{j in N}:
    TRx[j] == 0.5*(z[j] + w);	

subject to ConstraintSet10{j in N}:
    TCx[j] == Rx[j] - kappa; 	
	
subject to ConstraintSet12{j in N}:
    Rx[j] == sum{m in M} Y[j,m]*x[m];		
		